/*

6 Aug 07
- Need to add 0xffff direct logic to process_tx_queue
- Need to do initialise - done
- Need to add check calc to pkt_send - done

- Added pkt_send
- Added initialise

*/

#include "pic_packet.h"

static seen_packet    pkt_seen_list[PKT_SEEN_LIST_SIZE];
static sending_item   pkt_tx_queue[PKT_TX_QUEUE_SIZE];

uns8 pkt_seen_list_last = 0;
uns16 pkt_my_addr = 0x66;
uns16 pkt_my_next_pkt_id = 0;


uns8 pkt_seen(uns16 pkt_id, uns16 source_addr) {

uns8 count;

	for (count = 0; count < PKT_SEEN_LIST_SIZE; count++ ) {
		if ((pkt_seen_list[count].pkt_id == pkt_id) && 
		   (pkt_seen_list[count].source_addr == source_addr)) {
			return 1;
		} // if
	} // for
	return 0; // didn't find it
}			



uns8 pkt_load_rx(rf_packet *pkt_in) {

uns8 status = 0;
uns8 count;
uns16 orig_pkt_id;
uns8 ack_payload[4] = { 0xff, 0xff, 0xff, 0xff };
// was 669 and 16
rf_packet packet;

	memcpy(/*dst*/  (void *)&packet, // copy it in
		   /*src*/  (void *)pkt_in,
		   /*len*/  PKT_PACKET_SIZE);


    #ifdef PKT_DEBUG_HIGH
		serial_print_str("r");
        pkt_print_packet(&packet);
    #endif
    if (!pkt_check_check_byte(&packet)) {
		#ifdef PKT_DEBUG_HIGH
			serial_print_str("CF! ");
		#endif
		return PKT_STATUS_CHECK_FAIL;
	}	

	    //serial_print_str(" Rtr: ");
	    //serial_print_int(pkt->d.r1_addr);
		//serial_putc('/');
	    //serial_print_int(pkt->d.r2_addr);
		//serial_putc('/');
	    //serial_print_int(pkt->d.r3_addr);
		//serial_putc('/');
   	
		
	// Am I the dest addr?
	if (packet.d.dest_addr == pkt_my_addr) {
		status = PKT_STATUS_PKT_IS_FOR_ME;
		// It's for me. But is it an ack?
		if ((packet.d.payload[0] != 0xff) ||
			(packet.d.payload[1] != 0xff)) { // not an ack, so send one		
			//serial_print_str(" SACK");
			ack_payload[2] = packet.d.pkt_id & 0xff;
			ack_payload[3] = packet.d.pkt_id >> 8;
			pkt_send(packet.d.source_addr, ack_payload, PKT_FLAG_NO_RESEND);
		} else { // We got an ack!
			status = PKT_STATUS_PKT_IS_ACK_FOR_ME;
			orig_pkt_id = packet.d.payload[3];
			orig_pkt_id <<= 8;
			orig_pkt_id += packet.d.payload[2];
			//serial_print_str("O");
			//serial_print_int(orig_pkt_id);
			for (count = 0; count < PKT_TX_QUEUE_SIZE; count++) { // Search for a packet
				//serial_print_str("A");
				//serial_print_int(count);
				//serial_putc(' ');
				//serial_print_int(pkt_tx_queue[count].flag);
				//serial_putc(' ');
				//serial_print_int(pkt_tx_queue[count].packet.d.pkt_id);
				//serial_print_str("; ");
				if ((pkt_tx_queue[count].flag != PKT_FLAG_DELETED) &&
				(pkt_tx_queue[count].packet.d.pkt_id == orig_pkt_id)) {
					//serial_print_str(" FACK!");
					status = PKT_STATUS_PKT_IS_FACK_FOR_ME;	 // Found the ack
					pkt_tx_queue[count].flag = PKT_FLAG_DELETED;
				}
			}			
		} // [we got an ack]

		
		// If not seen before, then process		
		if (!pkt_seen(packet.d.pkt_id, packet.d.source_addr)) {
		
			// process payload
			pkt_process_payload(packet.d.source_addr, packet.d.pkt_id, packet.d.payload);
			// Add to seen list
			pkt_seen_list_last++;
			if (pkt_seen_list_last == PKT_SEEN_LIST_SIZE) {
				pkt_seen_list_last = 0;
			}
			pkt_seen_list[pkt_seen_list_last].pkt_id = packet.d.pkt_id;
			pkt_seen_list[pkt_seen_list_last].source_addr = packet.d.source_addr;
		} else {
			serial_print_str(" seen ");
		}	
	}		
	else
	// Is sender + pkt_id seen?
	if (pkt_seen(packet.d.pkt_id, packet.d.source_addr)) {
		// We've seen this packet before
		status = PKT_STATUS_SEEN_BEFORE;	
	} else 
	// Am I the sender?
	if (packet.d.source_addr == pkt_my_addr) {
		status = PKT_STATUS_I_AM_SENDER;
	} else
	// Is the r1_addr = 0xffff, meaning direct send?
	if (packet.d.r1_addr == 0xffff) {
		status = PKT_STATUS_DIRECT_SEND;
	} else 	 
	// Am I one of the routers?
	if ((packet.d.r1_addr == pkt_my_addr) || (packet.d.r2_addr == pkt_my_addr) ||
	   (packet.d.r3_addr == pkt_my_addr)) {
		status = PKT_STATUS_PREVIOUS_ROUTED_VIA_ME;
	} else 
	// Is r3 full?
	if (packet.d.r3_addr != 0) {
		status = PKT_STATUS_ROUTING_FULL;
	} else {  // I need to re-broadcast this packet
		status = PKT_STATUS_NEED_TO_REBROADCAST;
		// Add me as a router
		if (packet.d.r1_addr == 0) {
			packet.d.r1_addr = pkt_my_addr;
		} else if (packet.d.r2_addr == 0) {
			packet.d.r2_addr = pkt_my_addr;
		} else { 
			packet.d.r3_addr = pkt_my_addr;
		}		
		// Update check byte now we've changed packet
		pkt_calc_check_byte(&packet);
		// queue packet no resend
		pkt_queue_packet(&packet, PKT_FLAG_NO_RESEND);
	
		// add to seen list?
	} // [I need to re-broadcast this packet]

	return status;
}


	
uns8 pkt_queue_packet(rf_packet *packet, uns8 resend)
{
uns8 count, found;
	// find slot
	found = 0;
	for (count = 0; count < PKT_TX_QUEUE_SIZE; count++) {
		if (pkt_tx_queue[count].flag == PKT_FLAG_DELETED) {
			found = 1;
			break;
		}
	}
	if (found) {
	    #ifdef PKT_DEBUG_HIGH
			serial_print_str(" Qin ");
			serial_print_int(count);
		#endif	
		memcpy(/*dst*/  (void *)&pkt_tx_queue[count], // copy it in
			   /*src*/  (void *)packet,
			   /*len*/  PKT_PACKET_SIZE);
//    for (count = 0; count < PKT_PACKET_SIZE; count++) {
//	  pkt_tx_queue[count].packet.a[count] = packet->a[count];
//	}  

		// send count = 0
		if ((packet->d.r1_addr == 0)	// not routed
		    || (packet->d.r1_addr = 0xffff)) {	// or direct send
			found = 0;	// new packet
		} else {
			found = 2;  // resend, add some delay
		}
		pkt_tx_queue[count].sent_count = found;
		// resend = resend	
		pkt_tx_queue[count].flag = resend;
		return PKT_STATUS_QUEUED;
	} else {
		return PKT_STATUS_TX_QUEUE_FULL;
	}	
	
} // 138,8

void pkt_process_tx_queue() {

uns8 count;
uns16 current_tick;
uns8 sent_count;
uns8 flag;
    
    current_tick = tick_get_count();
    
	for (count = 0; count < PKT_TX_QUEUE_SIZE; count++) { // Search for a packet
		flag = pkt_tx_queue[count].flag;
		if (flag != PKT_FLAG_DELETED) {	// Got something to send
			#ifdef PKT_DEBUG_HIGH
				 serial_print_str(" SQ: ");
				 serial_print_int(count);
			#endif	
			sent_count = pkt_tx_queue[count].sent_count;
			if ((sent_count == 0) // Never tried to send 
			   || (tick_calc_diff(pkt_tx_queue[count].tick_sent, current_tick)
			         > PKT_RESEND_TICK_DELAY*sent_count)) { // Or more than delay time since last try

				// set tick count to current
				pkt_tx_queue[count].tick_sent = current_tick;
				pkt_send_packet((rf_packet*)&pkt_tx_queue[count]);
				if (flag == PKT_FLAG_NO_RESEND) {  // No resend?
					pkt_tx_queue[count].flag = PKT_FLAG_DELETED;  // Delete
					#ifdef PKT_DEBUG_HIGH
						serial_print_str(" SNR ");
						pkt_print_packet((rf_packet*)&pkt_tx_queue[count]);
					#endif	
					
				} else { //	Yes resend
					pkt_tx_queue[count].sent_count++;
					#ifdef PKT_DEBUG_HIGH
						printf("SC %d ", pkt_tx_queue[count].sent_count);
					#endif	
					
					if (sent_count +1 >= PKT_SEND_MAX_TRIES) { // Sent too many times
						pkt_tx_queue[count].flag = PKT_FLAG_DELETED; // Delete
					#ifdef PKT_DEBUG
						serial_print_str(" SF!\n ");
					#endif	
					}  // [Sent too many times]
					else { // clear direct send, next one goes to everyone!
						if (pkt_tx_queue[count].packet.d.r1_addr == 0xffff) {
							pkt_tx_queue[count].packet.d.r1_addr = 0;
						}	
					}	
				} // [Yes resend]		
			} // [Okay to send it]	
		}  // [Got something to send]
	} // [Search for a packet]
} // 269 + 12
	
	
void pkt_init(uns16 my_addr) {
	// go through tx queue and set all flags to DELETED	
uns8 count;
uns16 current_tick;

	pkt_my_addr = my_addr;

	for (count = 0; count < PKT_TX_QUEUE_SIZE; count++) {
		pkt_tx_queue[count].flag = PKT_FLAG_DELETED;  // Delete
	}	

	for (count = 0; count < PKT_SEEN_LIST_SIZE; count++ ) {
		pkt_seen_list[count].source_addr = 0xffff;
	}	
}	

	
uns8 pkt_send(uns16 dest_addr, uns8 *payload, uns8 resend) {

rf_packet my_packet;
uns8 result;
uns8 count;
//serial_print_str("\npkt send d:");
//serial_print_int(dest_addr);
//serial_print_str("s: ");
//serial_print_int(pkt_my_addr);
//serial_print_str("\n");
	my_packet.d.source_addr = pkt_my_addr;
	my_packet.d.pkt_id = pkt_my_next_pkt_id++;
    my_packet.d.dest_addr = dest_addr;
    my_packet.d.r1_addr = 0xffff; // for direct send
    my_packet.d.r2_addr = 0;
    my_packet.d.r3_addr = 0;
    for (count = 0; count < 8; count++) {
		my_packet.d.payload[count] = payload[count];
	}	
    pkt_calc_check_byte(&my_packet);
    
	#ifdef PKT_DEBUG_HIGH
		serial_print_str(" PS");
		pkt_print_packet(&my_packet);
	#endif
	//pkt_calc_check_byte(&my_packet);
    result =  pkt_queue_packet(&my_packet, resend);
    // pkt_process_tx_queue(); //-- just added this
}    
	
void pkt_calc_check_byte(rf_packet *packet) {
	
	uns8 calc, count;

	calc = 0;
	for (count = 0; count < PKT_PACKET_SIZE -1; count++) { // -1 since we don't want to include the check digit
		calc ^= packet->a[count];
	}
	packet->d.check_byte = calc;
}

uns8 pkt_check_check_byte(rf_packet *packet) {
	
	uns8 calc, count;

	calc = 0;
	for (count = 0; count < PKT_PACKET_SIZE -1; count++) { // -1 since we don't want to include the check digit
		calc ^= packet->a[count];
	}

	if (calc == packet->d.check_byte) {
		return 1;
	} else 
	    return 0;
	}    



uns8 pkt_print_packet(rf_packet *my_packet) {
	
	serial_print_str("[Pkt: s:");
	serial_print_int(my_packet->d.source_addr);
	serial_print_str(" i:");
	serial_print_int(my_packet->d.pkt_id);
    serial_print_str(" d:");
    serial_print_int(my_packet->d.dest_addr);
    serial_print_str(" r1:");
    serial_print_int(my_packet->d.r1_addr);
    serial_print_str(" r2:");
    serial_print_int(my_packet->d.r2_addr);
    serial_print_str(" r3:");
    serial_print_int(my_packet->d.r3_addr);
    serial_print_str(" p:");
    serial_print_int(my_packet->d.payload[0]);
    serial_putc(' ');
    serial_print_int(my_packet->d.payload[1]);
    serial_putc(' ');
    serial_print_int(my_packet->d.payload[2]);
    serial_putc(' ');
    serial_print_int(my_packet->d.payload[3]);
    serial_print_str("] ");
}    
	
