#ifndef __pic_packet_h
#define __pic_packet_h

#include "pic_utils.h"
#include "pic_serial.h"
#include "pic_tick.h"
#include "pic_serial.h"
#include "pic_tick.h"

#include "config.h"

#include <memory.h>

#ifdef PKT_USE_24L01
	#include "pic_rf_24l01.h"
#else
	#include "pic_rf_2401a.h"
#endif

#ifdef PKT_DEBUG_HIGH
	#ifndef PKT_DEBUG
		#define PKT_DEBUG
	#endif	
#endif	
// You'll need to define this routine in your own code:
//void pkt_process_payload(uns16 source_addr, uns16 pkt_id, uns8 *payload) {}

// Put this in your config.h:

// Config defines
/*
#define PKT_TX_QUEUE_SIZE 3
#define PKT_SEEN_LIST_SIZE 4
#define PKT_SEND_MAX_TRIES 2
#define PKT_RESEND_TICK_DELAY 500

#define PKT_USE_2401A
//#define PKT_USE_24L01


*/


// Status results

#define PKT_STATUS_SEEN_BEFORE				1
#define PKT_STATUS_I_AM_SENDER				2
#define PKT_STATUS_PKT_IS_FOR_ME			3
#define PKT_STATUS_PKT_IS_ACK_FOR_ME		4
#define PKT_STATUS_PKT_IS_FACK_FOR_ME		5
#define PKT_STATUS_DIRECT_SEND				6
#define PKT_STATUS_PREVIOUS_ROUTED_VIA_ME	7
#define PKT_STATUS_ROUTING_FULL				8
#define PKT_STATUS_NEED_TO_REBROADCAST		9
#define PKT_STATUS_QUEUED					10
#define PKT_STATUS_TX_QUEUE_FULL			11
#define PKT_STATUS_CHECK_FAIL				12

// flags

#define PKT_FLAG_NO_RESEND 0
#define PKT_FLAG_RESEND 1
#define PKT_FLAG_DELETED 0xff

// magic config address
#define PKT_CONFIG_ADDR 0xfffe

#define PKT_PACKET_SIZE 21

#define RF_RX_BUFFER_SIZE PKT_PACKET_SIZE
// Since we need to send address byte as well
#define RF_TX_BUFFER_SIZE PKT_PACKET_SIZE + 1
 
 
struct _rf_packet_det {  // 21 bytes
	 uns16 source_addr;
	 uns16 pkt_id;
	 uns16 dest_addr;
	 uns16 r1_addr;
	 uns16 r2_addr;
	 uns16 r3_addr;
	 uns8  payload[8];
	 uns8  check_byte;
};

typedef struct _rf_packet_det rf_packet_det;

union _rf_packet {
	rf_packet_det      d;
	char               a[PKT_PACKET_SIZE];
};
	
typedef union _rf_packet rf_packet;




struct _sending_item {
	rf_packet	packet;
    uns16     	tick_sent;
    uns8		sent_count;
    uns8		flag; // 0 don't resend / 1 resend / 0xff deleted
};

typedef struct _sending_item sending_item;

struct _seen_packet {
	uns16	pkt_id;
	uns16	source_addr;
};

typedef struct _seen_packet seen_packet;
	


uns8 pkt_load_rx(rf_packet* pkt);
uns8 pkt_rx_avail();
uns8 pkt_queue_packet(rf_packet *packet, uns8 resend);
void pkt_send_packet(rf_packet *packet);
void pkt_process_tx_queue();
void pkt_init(uns16 my_addr);
uns8 pkt_send(uns16 dest_addr, uns8 *payload, uns8 resend);
uns8 pkt_print_packet(rf_packet *my_packet);
void pkt_calc_check_byte(rf_packet *packet);
uns8 pkt_check_check_byte(rf_packet *packet); // 1= ok 0= bad
void pkt_process_payload(uns16 source_addr, uns16 pkt_id, uns8 *payload);

#ifdef PKT_USE_24L01
inline void pkt_send_packet(rf_packet *packet) {

//uns8 tx_buffer[PKT_PACKET_SIZE + 3]; // +3 for RF address (fixed)
uns8 count;

	#ifdef PKT_DEBUG
	    print_str(" <S>");
	#endif   
	#ifdef PKT_DEBUG_HIGH
	    pkt_print_packet(packet);
	#endif   
	//tx_buffer[0] = 0b11100111; // address
	//tx_buffer[1] = 0b11100111; // address
	//tx_buffer[2] = 0b11100111; // address
    //for (count = 0; count < PKT_PACKET_SIZE; count++) {
	 // tx_buffer[count+3] = packet->a[count];
	//}  
	//pic_rf_transmit(tx_buffer, PKT_PACKET_SIZE + 3);
	pic_rf_transmit((uns8 *)packet, PKT_PACKET_SIZE);
}

#else
inline void pkt_send_packet(rf_packet *packet) {

uns8 tx_buffer[PKT_PACKET_SIZE + 3]; // +3 for RF address (fixed)
uns8 count;

	#ifdef PKT_DEBUG
	    serial_print_str(" <S>");
	#endif    
	#ifdef PKT_DEBUG_HIGH
	    pkt_print_packet(packet);
	#endif   
	tx_buffer[0] = 0b11100111; // address
	tx_buffer[1] = 0b11100111; // address
	tx_buffer[2] = 0b11100111; // address
    for (count = 0; count < PKT_PACKET_SIZE; count++) {
	  tx_buffer[count+3] = packet->a[count];
	}  
	pic_rf_transmit(tx_buffer, PKT_PACKET_SIZE + 3);
	//pic_rf_transmit((uns8 *)packet, PKT_PACKET_SIZE);
}


#endif

#endif