#ifndef __pic_rf_2401a_h
#define __pic_rf_2401a_h defined

#include "config.h"
#include "pic_utils.h"

// Copy this into your config.h

/*  

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// pic_rf include
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define rf_ce_port PORTB
#define rf_ce_pin  1

#define rf_dr1_port PORTB
#define rf_dr1_pin  0

#define rf_cs_port PORTB
#define rf_cs_pin 3

#define rf_data_port PORTB
#define rf_data_pin 7

#define rf_clk1_port PORTB
#define rf_clk1_pin 6

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

*/



struct _rf_config {
	uns8 payload_width_ch2;
	uns8 payload_width_ch1;
	uns8 address_ch2[5];
	uns8 address_ch1[5];
	uns8 address_width; // (6 bits)
	uns8 crystal; 	// (3 bits)
	uns8 output_power; 	// (2 bits)
	uns8 channel; 	// (7 bits);
	uns8 options;
};	

typedef struct _rf_config rf_config; 

// rf_config options bits

#define ENABLE_RECEIVE		0
#define LONG_CRC		1
#define ENABLE_CRC		2

#define ENABLE_1_MBPS		5
#define ENABLE_SHOCKBURST	6
#define ENABLE_CH2		7

#define RECEIVE_MODE 1
#define TRANSMIT_MODE 0

static bit rf_current_mode_receive = 0;
static uns8 rf_current_channel = 2;



void pic_rf_send_byte(uns8 b);
void pic_rf_send_bytes(char *bytes, uns8 num_bytes);
void pic_rf_quick_init(char *my_config, uns8 my_channel, bit my_receive_on);

void pic_rf_setup();
void pic_rf_init(rf_config *my_config);
void pic_rf_send_bytes(char *bytes, uns8 num_bytes);
void pic_rf_send_byte(uns8 b);
void pic_rf_set_mode(uns8 mode);
void pic_rf_receive(uns8* data, uns8 bytes_to_receive);
void pic_rf_transmit(char* data, uns8 bytes_to_transmit);
void pic_rf_set_channel(uns8 channel);

inline void pic_rf_send_bytes_inline(char *bytes, uns8 num_bytes) {

uns8 byte_counter;
	for(byte_counter = 0 ; byte_counter < num_bytes ; byte_counter++) {
		pic_rf_send_byte(bytes[byte_counter]);
	}
}		

#define pic_rf_receive_mode() pic_rf_set_mode(RECEIVE_MODE)
#define pic_rf_transmit_mode() pic_rf_set_mode(TRANSMIT_MODE)

#define pic_rf_chip_select(value) change_pin(rf_cs_port, rf_cs_pin, value);
#define pic_rf_chip_enable(value) change_pin(rf_ce_port, rf_ce_pin, value);

#define pic_rf_init_inline(my_config) \
\
uns8 temp; \
uns8 options; \
	clear_bit(tris_array[rf_data_port - PORTA], rf_data_pin);  \
    clear_pin(rf_clk1_port, rf_clk1_pin);		 \
\
    pic_rf_chip_enable(0);\
    pic_rf_chip_select(1);	\
	\
	pic_rf_send_byte(my_config.payload_width_ch2);\
	pic_rf_send_byte(my_config.payload_width_ch1);\
	pic_rf_send_bytes(my_config.address_ch2, 5);\
	pic_rf_send_bytes(my_config.address_ch1, 5);\
	\
	\
	options = my_config.options;\
\
	temp = my_config.address_width << 2;\
	temp.1 = options.LONG_CRC; \
	temp.0 = options.ENABLE_CRC; \
\
	pic_rf_send_byte(temp);\
\
	temp = options & 0b11100000;  \
	temp |= (my_config.crystal & 0b00000111) << 2; \
	temp |= (my_config.output_power & 0b00000011); \
	\
	pic_rf_send_byte(temp);\
    	\
	temp = my_config.channel << 1;\
	rf_current_channel = my_config.channel;\
	\
	temp |= options.ENABLE_RECEIVE; \
	rf_current_mode_receive = options.ENABLE_RECEIVE;\
	\
	pic_rf_send_byte(temp);\
	\
    pic_rf_chip_select(0);	\
    pic_rf_chip_enable(1)	\
    \


// #define catches

#ifndef rf_ce_port
	#error "pic_rf_2401a: You haven't defined rf_ce_port in your config.h!"
#endif
#ifndef rf_ce_pin
	#error "pic_rfv: You haven't defined rf_ce_pin in your config.h!"
#endif

#ifndef rf_dr1_port
	#error "pic_rf_2401a: You haven't defined rf_dr1_port in your config.h!"
#endif
#ifndef rf_dr1_pin
	#error "pic_rf_2401a: You haven't defined rf_dr1_pin in your config.h!"
#endif

#ifndef rf_cs_port
	#error "pic_rf_2401a: You haven't defined rf_cs_port in your config.h!"
#endif
#ifndef rf_cs_pin
	#error "pic_rf_2401a: You haven't defined rf_cs_pin in your config.h!"
#endif

#ifndef rf_data_port
	#error "pic_rf_2401a: You haven't defined rf_data_port in your config.h!"
#endif
#ifndef rf_data_pin
	#error "pic_rf_2401a: You haven't defined rf_data_pin in your config.h!"
#endif

#ifndef rf_clk1_port
	#error "pic_rf_2401a: You haven't defined rf_clk1_port in your config.h!"
#endif
#ifndef rf_clk1_pin
	#error "pic_rf_2401a: You haven't defined rf_clk1_pin in your config.h!"
#endif

#endif
